function [inh,rc] = hessinv(hess0)
% Calculates "inverse" of hessian even when it is not positive definite

% Calculates "inverse" of hessian even when it is not positive definite
% Returns {hinv,rc}
% inh  - "inverted" value of hessian
% rc   - return code
%    0 : normal inverse
%    1 : used some methods to modify hessian before inversion
%    2 : return identity matrix

    %%%% local eval, evec, aeval, hess1, inh, inh1, inh2, p, rc; 
    [r,c]=size(hess0);
    if r~=c
        error('input should be a square matrix');
    end
    
    if ~all(isfinite(hess0))
        inh = eye(r);
        rc = 2; 
        return
    end
  

    rc = 0;    
    %warning off MATLAB:singularMatrix;
    %warning off MATLAB:nearlySingularMatrix
    inh = inv(hess0);
    if ~all(isfinite(inh))        
        %Method 1
        %First diagonalize hess0, then take the absolute value,
        %  and finally, rotate it back 
        [evec,eval] = eig(hess0);
        aeval = abs(eval);        
        %Method 1
        rc = 1;
        inh = inv(evec*aeval*evec');
        if ~all(isfinite(inh))
            %Method 1 without cordinate rotation 
            inh = inv(aeval);
            if ~all(isfinite(inh))
                %Method 2: return identity matrix
                inh = eye(r);
                rc = 2;                        
            end            
         end
    else
        diff1 = max(max(abs(hess0*inh-eye(r)))); 
        diff2 = max(max(abs(hess0*inh'-eye(r))));
        mdiff1 = max([diff1 diff2]);
        if mdiff1>0.01
            %The inversion was unstable
            %We try another way
            [evec,eval] = eig(hess0);
            eval_v = diag(eval,0);
            eps = 1e-10;
            new_v = (eval_v>=eps).*eval_v + (eval_v<eps)*eps;  %note:
                    %all negative will be changed!
            inh2 = evec*diag(1./new_v)*evec';

            % Sometimes (when the maxtrix is really badly conditioned) 
            diff3 = max(max(abs(hess0*inh2-eye(r))));
            diff4 = max(max(abs(hess0*inh2'-eye(r))));
            mdiff2 = max([diff3 diff4]);

            if mdiff1>mdiff2
                inh = inh2;
            end
        end
    end
    %warning on MATLAB:singularMatrix;
    %warning on MATLAB:nearlySingularMatrix

