function hh_generate_TS(obs, rho_x, rho_z, dev_s, dev_rho)
% Generates Hall-Horowitz Model data
%
% Generates Hall-Horowitz Model data
% Stores data in global variables hh_x and hh_z

global hh_x hh_z


s = 0.4; % H0: sigma = 0.4

x_AR1 = sqrt(1-rho_x^2)*generate_AR1(obs,rho_x);
z_AR1 = sqrt(1-rho_z^2)*generate_AR1(obs,rho_z);

hh_x = (0.4+dev_s)*(sqrt(1-dev_rho^2)*x_AR1+dev_rho*z_AR1); 
hh_z = s*z_AR1; 


function y = generate_AR1(N,rho)
  eps = randn(N,1);
  y = zeros(N,1);
  y(1) = eps(1)/sqrt(1-rho^2);
  for i=2:N
    y(i) = rho*y(i-1)+eps(i-1);
  end
