function [elval,elconv] = innerloop_HEL_disp(w)
%INNERLOOP_EL: calculates EL inner loop, w=f(theta)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    global el_max_innerloop_iterations el_printdiag el_use_testmax
    global el_max_step_len_dim 
    global el_tolerance_grad_direc
    global max_test max_lam_len max_val last_test
    max_test = 0; max_lam_len = 0; max_val=0;

    if length(el_printdiag)==0 
      el_printdiag = 1; 
    end

    if length(el_max_innerloop_iterations)==0
      el_max_innerloop_iterations = 100; %maximum number of iterations
           % IN the innerloop before it returns "Failed to converge". 
    end

    if length(el_use_testmax)==0;
      el_use_testmax = 0;
    end

    if length(el_max_step_len_dim )
      el_max_step_len_dim = 10; %maximum step length is 
         % el_max_step_len_dim*<number of momoments>
    end

    if length(el_tolerance_grad_direc)==0
      el_tolerance_grad_direc = .000000001;
    end

    el_min_step_mul = 0.01;
    
    el_infinity = 10^100; %used in outerloop to determine
            %whether the numbers used in calculations are too big
    el_max_moment_val = 10^8;
    global el_prob;

    if isbig(w,el_max_moment_val)
        elval=0;
        elconv=0;
        if el_printdiag ~= 0
          disp('HEL innerloop: m.c. values are too large.');
        end
        return
    end

    [n,cols] = size(w);    

    el_prob_2 = zeros(n,1);
    grad_tmp1 = zeros(n,cols);
    grad_tmp2 = zeros(n,cols);

    lam = zeros(cols,1);
    testmax = 2*.999999*n;
    iter = 1; 
    while iter<=el_max_innerloop_iterations;
        if el_printdiag ~= 0            
            disp(['HEL innerloop: iteration ' num2str(iter)]);
        end
        el_prob = 1./(1+w*lam);

        %Faster version:
        el_prob_2 = el_prob.^2;
        for j=1:cols
          grad_tmp1(:,j) = el_prob_2.*w(:,j);
          grad_tmp2(:,j) = el_prob.*w(:,j);
        end
        direc = 2*grad_tmp1'*grad_tmp2;        
        grad = -sum(grad_tmp1)';

        if isbig(direc,el_infinity)==0 && isbig(grad,el_infinity)==0
            if rcond(direc)<1e-15
              fprintf(1, 'HEL: rcond = %g\n', rcond(direc));
            end
            direc = direc\grad;
            test = 2*n*grad'*direc/sum(el_prob); 
            last_test = test;
            if test>max_test
              max_test = test;
            end
        else
            test = testmax + 1; %convergence criteria too large
        end
        
        direc_len = sqrt(sum(direc.^2));
        grad_len = sqrt(sum(grad.^2));
        if direc_len>0
          angle = grad'*direc/direc_len/grad_len;
        else
          angle = 0;
        end

        lam_len  = sqrt(sum(lam.^2));
        if lam_len>max_lam_len
          max_lam_len = lam_len;
        end

        if el_printdiag ~= 0
            disp(['test=' num2str(test) ' ang=' num2str(angle) ' dir_len=' num2str(direc_len) ' lam_len=' num2str(sqrt(sum(lam.^2))) ' el_val=' num2str(sum(el_prob)) ' lam=' num2str(lam(1))]); 
        end

        if direc_len > el_max_step_len_dim*cols 
            direc = direc/direc_len*el_max_step_len_dim*cols;
        end

        lik = -sum(el_prob);
        if -2*lik > max_val
          max_val = -2*lik;
        end

        if el_use_testmax && test > testmax
            if el_printdiag ~= 0
                disp('HEL innerloop: convergence criteria too large');
            end
            elval = 0;
            elconv = 0;            
            return
        end
     
        if test<0.00001
            elval = 2*n*(2-2*sqrt(sum(el_prob)/n));
            elval = 2*elval; %return the overid value
            elconv = 1;
            return
        end
       
        step = 1;        
        improved = 0;
        while step >= el_min_step_mul && improved == 0;
            lamtry = lam - step*direc;
            el_prob = 1.0 + (w*lamtry);        
            if any(el_prob <= 0) == 0 && -sum(1./el_prob) > lik;
                %checked positivity and the criterion got smaller
                improved = 1;
                lam = lamtry;
            else
                step = step/2;
            end           
        end;
        if improved==0
            if el_printdiag ~= 0
                disp('HEL innerloop: stepsize failed to find increase');
            end
            elval = 0;
            elconv = 0;   
            return
        end        
        iter = iter + 1;
    end

    if el_printdiag ~= 0
      disp('HEL innerloop: Maximum iterations reached');
    end

    elval = 0;
    elconv = 0;
end
