file_name_prefix = 'out\hh_polar';
numpoints = 64;
beta = 3;

iGMM2 = 1; iCUE = 2; iBEL = 3; iMHDE  = 4; iET = 5; num_methods = 5;
res_RMSE = zeros(numpoints, num_methods);
res_RMSE_T = zeros(numpoints, num_methods);
res_PR1 = zeros(numpoints, num_methods);
res_PR05 = zeros(numpoints, num_methods);
stat_dropped = zeros(numpoints,1);

for p_i=1:numpoints
  disp(sprintf('Processing point %3d out of %3d',p_i,numpoints));
  alpha = 2*pi()*(p_i-1)/numpoints;
  misspec_sd = 0.1*sin(alpha); 
  misspec_corr =  0.1*sqrt(2)*cos(alpha);
  
  load_file_name = [file_name_prefix int2str(p_i) '.mat'];
  S = load(load_file_name);
  
  if p_i==1
    this_set.obs = S.obs; %this is used to check that the parameters are the same in all files
    this_set.rho_x = S.rho_x;
    this_set.rho_z = S.rho_z;
    this_set.gel_block_size = S.gel_block_size;
    this_set.reps = length(S.res_arr);
    reps = length(S.res_arr);
  else
    if this_set.obs ~= S.obs || this_set.reps ~= length(S.res_arr) || this_set.rho_x ~= S.rho_x ||  ...
       this_set.rho_z ~= S.rho_z || this_set.gel_block_size ~= S.gel_block_size || ...
       abs(misspec_sd-S.misspec_sd)>0.0001 || abs(misspec_corr-S.misspec_corr)>0.0001
          error('non matching data??');
    end
  end
  
  failed = sum(S.res_failed,2);
  failed_num = sum(failed>0);
  res_f = sortrows([failed S.res_arr],1);
  est_res = res_f(1:reps-failed_num,2:num_methods+1);
  MEAN_bias = (mean(est_res)-beta);
  MED_bias = quantile(est_res,0.5)-beta;
  RMSE = sqrt(mean((est_res-beta).^2));
  trunc_c = 1;
  trunc = (est_res-beta).*(abs(est_res-beta)<trunc_c)...
     +trunc_c*(abs(est_res-beta)>=trunc_c).*sign(est_res-beta);
  RMSE_T = sqrt(mean(trunc.^2));
  MEAN_T = mean(trunc);
  PR1 = mean(abs(est_res-beta)>=1);
  PR05 = mean(abs(est_res-beta)>=0.5);
  PR075 = mean(abs(est_res-beta)>=0.75);
  PRX = mean(abs(est_res-beta)>=3);
  MAD = quantile(abs(est_res-beta),0.5);
  VAR = var(est_res);
  SD = std(est_res);

  res_RMSE(p_i,:)=RMSE;
  res_RMSE_T(p_i,:)=RMSE_T;
  res_PR1(p_i,:)=PR1;
  res_PR05(p_i,:)=PR05;
  res_PRX(p_i,:)=PRX;
end

alpha_grid = [(0:numpoints)'/numpoints]; 
plot_results_RMSE_T = [res_RMSE_T;  res_RMSE_T(1,:)];
plot_results_RMSE   = [res_RMSE;    res_RMSE(1,:)];
plot_results_PR1    = [res_PR1;     res_PR1(1,:)];
plot_results_PR05   = [res_PR05;    res_PR05(1,:)];
plot_results_PRX    = [res_PRX;     res_PRX(1,:)];

num_est_to_plot = 5;

colors = ['-b ','-r ','-g ','-k ','-c ']; %color
%colors = [':k ','--k','-.k','-ok','-dk']; %b/w

output_order = [iBEL, iMHDE, iET, iGMM2, iCUE];
legend_text_all = ['EL  ';'MHDE';'ET  ';'GMM2';'CUE '];
legend_text_no_cue=legend_text_all(1:4,:);
figure(1); 
subplot(2,2,1); newplot; hold on
for i=1:5
  plot(alpha_grid, plot_results_RMSE(:,output_order(i)), colors(3*(i-1)+1:3*i));
end
title('RMSE (all)');
hold off
subplot(2,2,2); newplot; hold on
for i=1:4
  plot(alpha_grid, plot_results_RMSE(:,output_order(i)), colors(3*(i-1)+1:3*i));
end
title('RMSE (no CUE)');
hold off
subplot(2,2,3); newplot; hold on
for i=1:5
  plot(alpha_grid, plot_results_PR05(:,output_order(i)), colors(3*(i-1)+1:3*i));
end
title('Pr (all)');
legend(legend_text_all,'Location','NorthWest');
hold off
subplot(2,2,4); newplot; hold on
for i=1:4
  plot(alpha_grid, plot_results_PR05(:,output_order(i)), colors(3*(i-1)+1:3*i));
end
title('Pr (no CUE)');
%title('PrX');
legend(legend_text_no_cue,'Location','NorthWest');
hold off
disp(sprintf('obs = %3d, rho_x = %2.2f, rho_z = %2.2f, blck_sz = %2d, reps = %4d', ...
  this_set.obs, this_set.rho_x, this_set.rho_z, this_set.gel_block_size, this_set.reps));

%% Formatting for the paper
plot_figure = figure(3);
newplot;
subplot(1,2,1);
axesRMSE = newplot;
hold off


h = plot(alpha_grid, plot_results_RMSE(:,output_order(1:4)), 'Parent',axesRMSE, ...
  'LineWidth',2, 'Color',[0 0 0]);
set(h(1),'Marker','.','LineStyle',':','DisplayName','EL');
set(h(2),'DisplayName','MHDE');
set(h(3),'LineStyle','--','DisplayName','ET');
set(h(4),'LineStyle','-.','DisplayName','GMM2');
set(axesRMSE, 'XTick',[0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1]);
set(axesRMSE, 'Position',[0.04 0.085 0.43 0.85]);

% Create title
title('RMSE');

% Create axes
subplot(1,2,2);
axesPR = newplot;
hold off

h = plot(alpha_grid, plot_results_PR05(:,output_order(1:5)), 'Parent',axesPR, ...
  'LineWidth',2, 'Color',[0 0 0]);
set(h(1),'Marker','.','LineStyle',':','DisplayName','EL');
set(h(2),'DisplayName','MHDE');
set(h(3),'LineStyle','--','DisplayName','ET');
set(h(4),'LineStyle','-.','DisplayName','GMM2');
set(h(5),'Marker','o','MarkerFaceColor','k', 'MarkerSize',4, 'DisplayName','CUE');
set(axesPR, 'XTick',[0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1]);
set(axesPR, 'Position',[0.53 0.085 0.43 0.85]);

% Create title
title('Pr');

% Create legend
legendRMSE = legend(axesRMSE,'show');
set(legendRMSE,'Position',[0.0815424979132499 0.717666330595022 0.127937336814621 0.203941730934019]);

% Create legend
legendPR = legend(axesPR,'show');
 

set(legendPR,'Position',[0.551441555154506 0.681803191055016 0.127937336814621 0.251928020565553]);
%set(legendPR, 'Position',[0.045 0.75 0.11 0.12]);


total_dropped = sum(stat_dropped);
[max_dropped, max_dropped_index] = max(stat_dropped);
fprintf('Total dropped %4d (%1.3f), max dropped %3d (%1.3f)', ...
  total_dropped, total_dropped/(numpoints*this_set.reps), ...
    max_dropped, max_dropped/this_set.reps);
