%DGP
obs = 100;
beta = 3;
rho_x = 0.0;
rho_z = 0.0;

b_min = 0.1;
b_max = 10;
num_points = 30;
delta = 0.001;

%results
iGMM2LR = 1;
iCUELR = 2;
iBEL = 3;
iMHDE  = 4;
iET = 5;
num_methods = 5;

res_arr = zeros(reps,num_methods);
res_failed = zeros(reps,num_methods);

global use_gel_speed gel_speed_conv gel_speed_mult gel_max_index 

gel_block_size = 1; %This is IID MC, so ignore all the "lr" suffixes!

tic
for rep = 1:reps
  hh_generate_TS(obs, rho_x, rho_z, misspec_sd, misspec_corr);

  [b_gmm1, b_gmm2] = gmm12lr(@hh_mc,gel_block_size, b_min,b_max,num_points,delta); 
  res_arr(rep,iGMM2LR) = b_gmm2; % use GMM2
  if gel_block_size>1
    error('The above is for the IID case only!!');
  end

  b_cue_lr = cue_lr(@hh_mc,gel_block_size, b_min,b_max,num_points,delta);
  res_arr(rep,iCUELR) = b_cue_lr;

  use_gel_speed = 0; %NOT USED
  gel_speed_mult = 1/(delta/2);
  gel_max_index = b_max*gel_speed_mult;
  gel_speed_conv = -ones(gel_max_index,1);

  % *** EL ***
  [b_bel,val] = BEL_line(@hh_mc, gel_block_size, 1, 'EL', b_min, b_max, num_points, delta );
  res_arr(rep,iBEL) = b_bel;
  res_failed(rep,iBEL) = val>10^10;

  % *** MHDE ***
  [b_bel,val] = BEL_line(@hh_mc, gel_block_size, 1, 'MHDE', b_min, b_max, num_points, delta );
  res_arr(rep,iMHDE) = b_bel;
  res_failed(rep,iMHDE) = val>10^10;

  % *** ET ***
  [b_bel,val] = BEL_line(@hh_mc, gel_block_size, 1, 'ET', b_min, b_max, num_points, delta );
  res_arr(rep,iET) = b_bel;
  res_failed(rep,iET) = val>10^10;

  if ~exist('display_iter_info_flag') || display_iter_info_flag
    disp(sprintf('rep = %3.0f, bgmm2  = %1.3f, bel1  = %1.3f, bmhde1  = %1.3f, bet1  = %1.3f',rep, ...
       res_arr(rep,iGMM2LR), res_arr(rep,iBEL), res_arr(rep,iMHDE), res_arr(rep,iET)));
  end
end
time_took_slave = toc

if ~isempty(slave_file_name)
  save(slave_file_name,'obs','rho_x','rho_z','gel_block_size','misspec_sd','misspec_corr','res_arr','res_failed');
end

%print results
%results
failed = sum(res_failed,2);
failed_num = sum(failed>0);
res_f = sortrows([failed res_arr],1);
est_res = res_f(1:reps-failed_num,2:num_methods+1);
MEAN_bias = (mean(est_res)-beta);
MED_bias = quantile(est_res,0.5)-beta;
RMSE = sqrt(mean((est_res-beta).^2));
trunc_c = 1;
trunc = (est_res-beta).*(abs(est_res-beta)<trunc_c)...
   +trunc_c*(abs(est_res-beta)>=trunc_c).*sign(est_res-beta);
RMSE_T = sqrt(mean(trunc.^2));
MEAN_T = mean(trunc);
PR1 = mean(abs(est_res-beta)>=1);
MAD = quantile(abs(est_res-beta),0.5);
VAR = var(est_res);
SD = std(est_res);

disp_est_res = [(1:num_methods)' MEAN_bias' MED_bias' RMSE' MAD' RMSE_T' PR1' SD' sum(res_failed)'];
disp('     Meth     MEAN_B    MED_B      RMSE      MAD      RMSE_T    PR1    SD  Failed');
disp(disp_est_res);
disp(sprintf('Valid reps: %3.0f; failed reps: %3.0f',reps-failed_num,failed_num));
toc 

out_indices = [iBEL, iMHDE, iET, iGMM2LR, iCUELR];
res_RMSE_T = RMSE(out_indices);
res_PR1    = PR1(out_indices)';

